@@ -58,14 +58,16 @@ mc_skin_get_list_from_dir (const gchar * base_dir, GPtrArray * list)
 
     if (dir != NULL)
     {
-        while ((name = (gchar *) g_dir_read_name (dir)) != NULL)
+        const gchar *cname;
+
+        while ((cname = g_dir_read_name (dir)) != NULL)
         {
             gchar *sname;
             size_t slen;
             unsigned int i;
 
-            slen = strlen (name);
-            sname = g_strndup (name, slen);
+            slen = strlen (cname);
+            sname = g_strndup (cname, slen);
 
             if (slen > 4 && strcmp (sname + slen - 4, ".ini") == 0)
                 sname[slen - 4] = '\0';
@@ -89,10 +91,7 @@ mc_skin_get_list_from_dir (const gchar * base_dir, GPtrArray * list)
 static int
 string_array_comparator (gconstpointer a, gconstpointer b)
 {
-    char *aa = *(char **) a;
-    char *bb = *(char **) b;
-
-    return strcmp (aa, bb);
+    return strcmp (*(char *const *) a, *(char *const *) b);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -102,7 +101,7 @@ mc_skin_ini_file_load_search_in_dir (mc_skin_t * mc_skin, const gchar * base_dir
 {
     char *file_name, *file_name2;
 
-    file_name = g_build_filename (base_dir, MC_SKINS_SUBDIR, mc_skin->name, NULL);
+    file_name = g_build_filename (base_dir, MC_SKINS_SUBDIR, mc_skin->name, (char *) NULL);
     if (exist_file (file_name))
     {
         mc_skin->config = mc_config_init (file_name, TRUE);
@@ -112,7 +111,7 @@ mc_skin_ini_file_load_search_in_dir (mc_skin_t * mc_skin, const gchar * base_dir
     g_free (file_name);
 
     file_name2 = g_strdup_printf ("%s.ini", mc_skin->name);
-    file_name = g_build_filename (base_dir, MC_SKINS_SUBDIR, file_name2, NULL);
+    file_name = g_build_filename (base_dir, MC_SKINS_SUBDIR, file_name2, (char *) NULL);
     g_free (file_name2);
 
     if (exist_file (file_name))
