@@ -2,9 +2,8 @@
    Color setup.
    Interface functions.
 
-   Copyright (C) 1994, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-   2007, 2008, 2009, 2010, 2011
-   The Free Software Foundation, Inc.
+   Copyright (C) 1994-2016
+   Free Software Foundation, Inc.
 
    Written by:
    Andrew Borodin <aborodin@vmail.ru>, 2009
