@@ -50,13 +50,6 @@
 
 /*** global variables ****************************************************************************/
 
-#ifdef HAVE_SLANG
-/* If true program softkeys (HP terminals only) on startup and after every
-   command ran in the subshell to the description found in the termcap/terminfo
-   database */
-int reset_hp_softkeys = 0;
-#endif
-
 int mc_tty_frm[MC_TTY_FRM_MAX];
 
 /*** file scope macro definitions ****************************************************************/
