@@ -1,7 +1,7 @@
 /*
    Widgets for the Midnight Commander
 
-   Copyright (C) 1994-2014
+   Copyright (C) 1994-2016
    Free Software Foundation, Inc.
 
    Authors:
@@ -10,7 +10,7 @@
    Jakub Jelinek, 1995
    Andrej Borsenkow, 1996
    Norbert Warmuth, 1997
-   Andrew Borodin <aborodin@vmail.ru>, 2009, 2010, 2013
+   Andrew Borodin <aborodin@vmail.ru>, 2009-2016
 
    This file is part of the Midnight Commander.
 
@@ -37,12 +37,10 @@
 #include <stdlib.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <fcntl.h>
 
 #include "lib/global.h"
 
 #include "lib/tty/tty.h"
-#include "lib/tty/mouse.h"
 #include "lib/tty/key.h"        /* XCTRL and ALT macros  */
 #include "lib/fileloc.h"
 #include "lib/skin.h"
@@ -56,7 +54,7 @@
 
 /*** global variables ****************************************************************************/
 
-int quote = 0;
+gboolean quote = FALSE;
 
 const global_keymap_t *input_map = NULL;
 
@@ -93,7 +91,7 @@ get_history_length (const GList * history)
 {
     size_t len = 0;
 
-    for (; history != NULL; history = g_list_previous (history))
+    for (; history != NULL; history = (const GList *) g_list_previous (history))
         len++;
 
     return len;
@@ -127,27 +125,10 @@ draw_history_button (WInput * in)
 
 /* --------------------------------------------------------------------------------------------- */
 
-static void
-input_set_markers (WInput * in, long m1)
-{
-    in->mark = m1;
-}
-
-/* --------------------------------------------------------------------------------------------- */
-
 static void
 input_mark_cmd (WInput * in, gboolean mark)
 {
-    if (mark == 0)
-    {
-        in->highlight = FALSE;
-        input_set_markers (in, 0);
-    }
-    else
-    {
-        in->highlight = TRUE;
-        input_set_markers (in, in->point);
-    }
+    in->mark = mark ? in->point : -1;
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -155,17 +136,15 @@ input_mark_cmd (WInput * in, gboolean mark)
 static gboolean
 input_eval_marks (WInput * in, long *start_mark, long *end_mark)
 {
-    if (in->highlight)
+    if (in->mark >= 0)
     {
-        *start_mark = min (in->mark, in->point);
-        *end_mark = max (in->mark, in->point);
+        *start_mark = MIN (in->mark, in->point);
+        *end_mark = MAX (in->mark, in->point);
         return TRUE;
     }
-    else
-    {
-        *start_mark = *end_mark = 0;
-        return FALSE;
-    }
+
+    *start_mark = *end_mark = -1;
+    return FALSE;
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -173,8 +152,8 @@ input_eval_marks (WInput * in, long *start_mark, long *end_mark)
 static void
 delete_region (WInput * in, int x_first, int x_last)
 {
-    int first = min (x_first, x_last);
-    int last = max (x_first, x_last);
+    int first = MIN (x_first, x_last);
+    int last = MAX (x_first, x_last);
     size_t len;
 
     input_mark_cmd (in, FALSE);
@@ -244,7 +223,7 @@ input_history_strip_password (char *url)
         return g_strdup (url);
     *colon = '\0';
 
-    return g_strconcat (url, at, NULL);
+    return g_strconcat (url, at, (char *) NULL);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -315,13 +294,11 @@ static cb_ret_t
 insert_char (WInput * in, int c_code)
 {
     int res;
+    long m1, m2;
+
+    if (input_eval_marks (in, &m1, &m2))
+        delete_region (in, m1, m2);
 
-    if (in->highlight)
-    {
-        long m1, m2;
-        if (input_eval_marks (in, &m1, &m2))
-            delete_region (in, m1, m2);
-    }
     if (c_code == -1)
         return MSG_NOT_HANDLED;
 
@@ -443,13 +420,14 @@ forward_word (WInput * in)
 static void
 backward_word (WInput * in)
 {
-    const char *p, *p_tmp;
+    const char *p;
 
-    for (p = in->buffer + str_offset_to_pos (in->buffer, in->point);
-         (p != in->buffer) && (p[0] == '\0'); str_cprev_char (&p), in->point--);
+    p = in->buffer + str_offset_to_pos (in->buffer, in->point);
 
     while (p != in->buffer)
     {
+        const char *p_tmp;
+
         p_tmp = p;
         str_cprev_char (&p);
         if (!str_isspace (p) && !str_ispunct (p))
@@ -508,8 +486,8 @@ delete_char (WInput * in)
 static void
 copy_region (WInput * in, int x_first, int x_last)
 {
-    int first = min (x_first, x_last);
-    int last = max (x_first, x_last);
+    int first = MIN (x_first, x_last);
+    int last = MAX (x_first, x_last);
 
     if (last == first)
     {
@@ -603,8 +581,7 @@ clear_line (WInput * in)
     in->need_push = TRUE;
     in->buffer[0] = '\0';
     in->point = 0;
-    in->mark = 0;
-    in->highlight = FALSE;
+    in->mark = -1;
     in->charpoint = 0;
 }
 
@@ -701,30 +678,34 @@ port_region_marked_for_delete (WInput * in)
 /* --------------------------------------------------------------------------------------------- */
 
 static cb_ret_t
-input_execute_cmd (WInput * in, unsigned long command)
+input_execute_cmd (WInput * in, long command)
 {
     cb_ret_t res = MSG_HANDLED;
 
-    /* a highlight command like shift-arrow */
-    if (command == CK_MarkLeft || command == CK_MarkRight ||
-        command == CK_MarkToWordBegin || command == CK_MarkToWordEnd ||
-        command == CK_MarkToHome || command == CK_MarkToEnd)
+    switch (command)
     {
-        if (!in->highlight)
+    case CK_MarkLeft:
+    case CK_MarkRight:
+    case CK_MarkToWordBegin:
+    case CK_MarkToWordEnd:
+    case CK_MarkToHome:
+    case CK_MarkToEnd:
+        /* a highlight command like shift-arrow */
+        if (in->mark < 0)
         {
             input_mark_cmd (in, FALSE); /* clear */
             input_mark_cmd (in, TRUE);  /* marking on */
         }
-    }
-
-    switch (command)
-    {
+        break;
     case CK_WordRight:
     case CK_WordLeft:
     case CK_Right:
     case CK_Left:
-        if (in->highlight)
+        if (in->mark >= 0)
             input_mark_cmd (in, FALSE);
+        break;
+    default:
+        break;
     }
 
     switch (command)
@@ -754,26 +735,27 @@ input_execute_cmd (WInput * in, unsigned long command)
         forward_word (in);
         break;
     case CK_BackSpace:
-        if (in->highlight)
         {
             long m1, m2;
+
             if (input_eval_marks (in, &m1, &m2))
                 delete_region (in, m1, m2);
+            else
+                backward_delete (in);
         }
-        else
-            backward_delete (in);
         break;
     case CK_Delete:
         if (in->first)
             port_region_marked_for_delete (in);
-        else if (in->highlight)
+        else
         {
             long m1, m2;
+
             if (input_eval_marks (in, &m1, &m2))
                 delete_region (in, m1, m2);
+            else
+                delete_char (in);
         }
-        else
-            delete_char (in);
         break;
     case CK_DeleteToWordEnd:
         kill_word (in);
@@ -785,7 +767,7 @@ input_execute_cmd (WInput * in, unsigned long command)
         input_mark_cmd (in, TRUE);
         break;
     case CK_Remove:
-        delete_region (in, in->point, in->mark);
+        delete_region (in, in->point, MAX (in->mark, 0));
         break;
     case CK_DeleteToEnd:
         kill_line (in);
@@ -794,11 +776,16 @@ input_execute_cmd (WInput * in, unsigned long command)
         clear_line (in);
         break;
     case CK_Store:
-        copy_region (in, in->mark, in->point);
+        copy_region (in, MAX (in->mark, 0), in->point);
         break;
     case CK_Cut:
-        copy_region (in, in->mark, in->point);
-        delete_region (in, in->point, in->mark);
+        {
+            long m;
+
+            m = MAX (in->mark, 0);
+            copy_region (in, m, in->point);
+            delete_region (in, in->point, m);
+        }
         break;
     case CK_Yank:
         yank (in);
@@ -822,10 +809,20 @@ input_execute_cmd (WInput * in, unsigned long command)
         res = MSG_NOT_HANDLED;
     }
 
-    if (command != CK_MarkLeft && command != CK_MarkRight &&
-        command != CK_MarkToWordBegin && command != CK_MarkToWordEnd &&
-        command != CK_MarkToHome && command != CK_MarkToEnd)
-        in->highlight = FALSE;
+    switch (command)
+    {
+    case CK_MarkLeft:
+    case CK_MarkRight:
+    case CK_MarkToWordBegin:
+    case CK_MarkToWordEnd:
+    case CK_MarkToHome:
+    case CK_MarkToEnd:
+        /* do nothing */
+        break;
+    default:
+        in->mark = -1;
+        break;
+    }
 
     return res;
 }
@@ -906,56 +903,68 @@ input_destroy (WInput * in)
     }
     g_free (in->history.name);
     g_free (in->buffer);
-
-    g_free (kill_buffer);
-    kill_buffer = NULL;
+    MC_PTR_FREE (kill_buffer);
 }
 
 /* --------------------------------------------------------------------------------------------- */
 
+/**
+ * Calculates the buffer index (aka "point") corresponding to some screen coordinate.
+ */
 static int
-input_event (Gpm_Event * event, void *data)
+input_screen_to_point (const WInput * in, int x)
 {
-    WInput *in = INPUT (data);
-    Widget *w = WIDGET (data);
+    x += in->term_first_shown;
 
-    if (!mouse_global_in_widget (event, w))
-        return MOU_UNHANDLED;
+    if (x < 0)
+        return 0;
 
-    if ((event->type & GPM_DOWN) != 0)
+    if (x < str_term_width1 (in->buffer))
+        return str_column_to_pos (in->buffer, x);
+
+    return str_length (in->buffer);
+}
+
+/* --------------------------------------------------------------------------------------------- */
+
+static void
+input_mouse_callback (Widget * w, mouse_msg_t msg, mouse_event_t * event)
+{
+    /* save point between MSG_MOUSE_DOWN and MSG_MOUSE_DRAG */
+    static int prev_point = 0;
+    WInput *in = INPUT (w);
+
+    switch (msg)
     {
-        in->first = FALSE;
-        input_mark_cmd (in, FALSE);
-    }
-
-    if ((event->type & (GPM_DOWN | GPM_DRAG)) != 0)
-    {
-        Gpm_Event local;
-
-        local = mouse_get_local (event, w);
-
+    case MSG_MOUSE_DOWN:
         dlg_select_widget (w);
+        in->first = FALSE;
 
-        if (local.x >= w->cols - HISTORY_BUTTON_WIDTH + 1 && should_show_history_button (in))
+        if (event->x >= w->cols - HISTORY_BUTTON_WIDTH && should_show_history_button (in))
             do_show_hist (in);
         else
         {
-            in->point = str_length (in->buffer);
-            if (local.x + in->term_first_shown - 1 < str_term_width1 (in->buffer))
-                in->point = str_column_to_pos (in->buffer, local.x + in->term_first_shown - 1);
+            input_mark_cmd (in, FALSE);
+            input_set_point (in, input_screen_to_point (in, event->x));
+            /* save point for the possible following MSG_MOUSE_DRAG action */
+            prev_point = in->point;
         }
+        break;
 
-        input_update (in, TRUE);
+    case MSG_MOUSE_DRAG:
+        /* start point: set marker using point before first MSG_MOUSE_DRAG action */
+        if (in->mark < 0)
+            in->mark = prev_point;
+
+        input_set_point (in, input_screen_to_point (in, event->x));
+        break;
+
+    default:
+        /* don't create highlight region of 0 length */
+        if (in->mark == in->point)
+            input_mark_cmd (in, FALSE);
+        break;
     }
-
-    /* A lone up mustn't do anything */
-    if (in->highlight && (event->type & (GPM_UP | GPM_DRAG)) != 0)
-        return MOU_NORMAL;
-
-    if ((event->type & GPM_DRAG) == 0)
-        input_mark_cmd (in, TRUE);
-
-    return MOU_NORMAL;
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -1000,13 +1009,13 @@ input_new (int y, int x, const int *colors, int width, const char *def_text,
 
     in = g_new (WInput, 1);
     w = WIDGET (in);
-    widget_init (w, y, x, 1, width, input_callback, input_event);
+    widget_init (w, y, x, 1, width, input_callback, input_mouse_callback);
     w->options |= W_IS_INPUT;
     w->set_options = input_set_options_callback;
 
     in->color = colors;
     in->first = TRUE;
-    in->highlight = FALSE;
+    in->mark = -1;
     in->term_first_shown = 0;
     in->disable_update = 0;
     in->is_password = FALSE;
@@ -1061,9 +1070,9 @@ input_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *d
     case MSG_KEY:
         if (parm == XCTRL ('q'))
         {
-            quote = 1;
+            quote = TRUE;
             v = input_handle_char (in, ascii_alpha_to_cntrl (tty_getch ()));
-            quote = 0;
+            quote = FALSE;
             return v;
         }
 
@@ -1075,9 +1084,9 @@ input_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *d
         /* When pasting multiline text, insert literal Enter */
         if ((parm & ~KEY_M_MASK) == '\n')
         {
-            quote = 1;
+            quote = TRUE;
             v = input_handle_char (in, '\n');
-            quote = 0;
+            quote = FALSE;
             return v;
         }
 
@@ -1127,14 +1136,14 @@ cb_ret_t
 input_handle_char (WInput * in, int key)
 {
     cb_ret_t v;
-    unsigned long command;
+    long command;
 
-    if (quote != 0)
+    if (quote)
     {
         input_free_completions (in);
         v = insert_char (in, key);
         input_update (in, TRUE);
-        quote = 0;
+        quote = FALSE;
         return v;
     }
 
@@ -1171,7 +1180,7 @@ input_handle_char (WInput * in, int key)
 int
 input_key_is_in_map (WInput * in, int key)
 {
-    unsigned long command;
+    long command;
 
     (void) in;
 
@@ -1194,12 +1203,12 @@ input_assign_text (WInput * in, const char *text)
         text = "";
 
     input_free_completions (in);
-    in->mark = 0;
+    in->mark = -1;
     in->need_push = TRUE;
     in->charpoint = 0;
 
     text_len = strlen (text);
-    buffer_len = 1 + max ((size_t) w->cols, text_len);
+    buffer_len = 1 + MAX ((size_t) w->cols, text_len);
     in->current_max_size = buffer_len;
     if (buffer_len > (size_t) w->cols)
         in->buffer = g_realloc (in->buffer, buffer_len);
@@ -1210,6 +1219,14 @@ input_assign_text (WInput * in, const char *text)
 
 /* --------------------------------------------------------------------------------------------- */
 
+gboolean
+input_is_empty (const WInput * in)
+{
+    return (in == NULL || in->buffer == NULL || in->buffer[0] == '\0');
+}
+
+/* --------------------------------------------------------------------------------------------- */
+
 /* Inserts text in input line */
 void
 input_insert (WInput * in, const char *text, gboolean insert_extra_space)
@@ -1231,7 +1248,7 @@ input_set_point (WInput * in, int pos)
     int max_pos;
 
     max_pos = str_length (in->buffer);
-    pos = min (pos, max_pos);
+    pos = MIN (pos, max_pos);
     if (pos != in->point)
         input_free_completions (in);
     in->point = pos;
@@ -1263,7 +1280,7 @@ input_update (WInput * in, gboolean clear_first)
     buf_len = str_length (in->buffer);
 
     /* Adjust the mark */
-    in->mark = min (in->mark, buf_len);
+    in->mark = MIN (in->mark, buf_len);
 
     pw = str_term_width2 (in->buffer, in->point);
 
@@ -1289,7 +1306,7 @@ input_update (WInput * in, gboolean clear_first)
 
     if (!in->is_password)
     {
-        if (!in->highlight)
+        if (in->mark < 0)
             tty_print_string (str_term_substring (in->buffer, in->term_first_shown,
                                                   w->cols - has_history));
         else
@@ -1316,7 +1333,7 @@ input_update (WInput * in, gboolean clear_first)
                     widget_move (in, 0, m1 - in->term_first_shown);
                     buf_width = str_term_width2 (in->buffer, m1);
                     sel_width =
-                        min (m2 - m1, (w->cols - has_history) - (buf_width - in->term_first_shown));
+                        MIN (m2 - m1, (w->cols - has_history) - (buf_width - in->term_first_shown));
                     tty_print_string (str_term_substring (in->buffer, m1, sel_width));
                 }
             }
@@ -1375,8 +1392,7 @@ input_clean (WInput * in)
     in->buffer[0] = '\0';
     in->point = 0;
     in->charpoint = 0;
-    in->mark = 0;
-    in->highlight = FALSE;
+    in->mark = -1;
     input_free_completions (in);
     input_update (in, FALSE);
 }
