@@ -40,6 +40,7 @@ typedef struct WLEntry
     char *text;                 /* Text to display */
     int hotkey;
     void *data;                 /* Client information */
+    gboolean free_data;         /* Whether to free the data on entry's removal */
 } WLEntry;
 
 typedef struct WListbox
@@ -51,6 +52,7 @@ typedef struct WListbox
     gboolean allow_duplicates;  /* Do we allow duplicates on the list? */
     gboolean scrollbar;         /* Draw a scrollbar? */
     gboolean deletable;         /* Can list entries be deleted? */
+    gboolean focused;           /* Listbox is focused */
     lcback_fn callback;         /* The callback function */
     int cursor_x, cursor_y;     /* Cache the values */
 } WListbox;
@@ -73,8 +75,8 @@ void listbox_remove_current (WListbox * l);
 gboolean listbox_is_empty (const WListbox * l);
 void listbox_set_list (WListbox * l, GList * list);
 void listbox_remove_list (WListbox * l);
-char *listbox_add_item (WListbox * l, listbox_append_t pos,
-                        int hotkey, const char *text, void *data);
+char *listbox_add_item (WListbox * l, listbox_append_t pos, int hotkey, const char *text,
+                        void *data, gboolean free_data);
 
 /*** inline functions ****************************************************************************/
 
