@@ -46,14 +46,15 @@ typedef enum
     RECURSIVE_ABORT = 4
 } FileCopyMode;
 
+/* ATTENTION: avoid overlapping with B_* values (lib/widget/dialog.h) */
 typedef enum
 {
-    FILE_CONT = 0,
-    FILE_RETRY = 1,
-    FILE_SKIP = 2,
-    FILE_ABORT = 3,
-    FILE_SKIPALL = 4,
-    FILE_SUSPEND = 5
+    FILE_CONT = 10,
+    FILE_RETRY,
+    FILE_SKIP,
+    FILE_ABORT,
+    FILE_SKIPALL,
+    FILE_SUSPEND
 } FileProgressStatus;
 
 /* First argument passed to real functions */
@@ -128,7 +129,7 @@ typedef struct
     gboolean preserve_uidgid;
 
     /* The bits to preserve in created files' modes on file copy */
-    int umask_kill;
+    mode_t umask_kill;
 
     /* The mask of files to actually operate on */
     char *dest_mask;
@@ -165,6 +166,7 @@ typedef struct
 typedef struct
 {
     size_t progress_count;
+    size_t prev_progress_count; /* Used in OP_MOVE between copy and remove directories */
     uintmax_t progress_bytes;
     uintmax_t copied_bytes;
     size_t bps;
@@ -173,7 +175,7 @@ typedef struct
     double eta_secs;
 
     gboolean ask_overwrite;
-} FileOpTotalContext;
+} file_op_total_context_t;
 
 /*** global variables defined in .c file *********************************************************/
 
@@ -184,8 +186,8 @@ extern const char *op_names[3];
 file_op_context_t *file_op_context_new (FileOperation op);
 void file_op_context_destroy (file_op_context_t * ctx);
 
-FileOpTotalContext *file_op_total_context_new (void);
-void file_op_total_context_destroy (FileOpTotalContext * tctx);
+file_op_total_context_t *file_op_total_context_new (void);
+void file_op_total_context_destroy (file_op_total_context_t * tctx);
 
 /* The following functions are implemented separately by each port */
 FileProgressStatus file_progress_real_query_replace (file_op_context_t * ctx,
